package net.retiolus.osm2gmaps.activities

import android.content.ClipData
import android.content.ClipboardManager
import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.content.pm.PackageManager
import android.location.Location
import android.location.LocationManager
import android.net.Uri
import android.os.Bundle
import android.util.Log
import android.widget.Button
import android.widget.EditText
import android.widget.Toast
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.android.material.textfield.TextInputEditText
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.DelicateCoroutinesApi
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import net.retiolus.osm2gmaps.R
import net.retiolus.osm2gmaps.adapters.ConvertedLinksAdapter
import net.retiolus.osm2gmaps.utils.PreferencesUtil
import net.retiolus.osm2gmaps.utils.maps.LinkWithTitle
import net.retiolus.osm2gmaps.utils.maps.LocationInfo
import net.retiolus.osm2gmaps.utils.maps.converters.create2GisLink
import net.retiolus.osm2gmaps.utils.maps.converters.createAcmeMapperLink
import net.retiolus.osm2gmaps.utils.maps.converters.createAppleMapsLink
import net.retiolus.osm2gmaps.utils.maps.converters.createBingMapsLink
import net.retiolus.osm2gmaps.utils.maps.converters.createCopernixLink
import net.retiolus.osm2gmaps.utils.maps.converters.createDMSLink
import net.retiolus.osm2gmaps.utils.maps.converters.createGeoHackLink
import net.retiolus.osm2gmaps.utils.maps.converters.createGeoLink
import net.retiolus.osm2gmaps.utils.maps.converters.createGoogleMapsLink
import net.retiolus.osm2gmaps.utils.maps.converters.createHereWeGoLink
import net.retiolus.osm2gmaps.utils.maps.converters.createMagicEarthLink
import net.retiolus.osm2gmaps.utils.maps.converters.createMapQuestLink
import net.retiolus.osm2gmaps.utils.maps.converters.createOmapsAppLink
import net.retiolus.osm2gmaps.utils.maps.converters.createOpenHistoricalMapLink
import net.retiolus.osm2gmaps.utils.maps.converters.createOpenStreetMapLink
import net.retiolus.osm2gmaps.utils.maps.converters.createOpenTopoMapLink
import net.retiolus.osm2gmaps.utils.maps.converters.createWazeLink
import net.retiolus.osm2gmaps.utils.maps.converters.createWikiMapiaConverterLink
import net.retiolus.osm2gmaps.utils.maps.converters.createWikiMiniAtlasLink
import net.retiolus.osm2gmaps.utils.maps.converters.createYandexMapsLink
import net.retiolus.osm2gmaps.utils.maps.converters.decodeOmapsAppShortCode
import net.retiolus.osm2gmaps.utils.maps.converters.decodeOsmGoShortCode
import net.retiolus.osm2gmaps.utils.maps.converters.extract2GisCoordinates
import net.retiolus.osm2gmaps.utils.maps.converters.extractAppleMapsCoordinates
import net.retiolus.osm2gmaps.utils.maps.converters.extractBingMapsCoordinates
import net.retiolus.osm2gmaps.utils.maps.converters.extractMagicEarhCoordinates
import net.retiolus.osm2gmaps.utils.maps.converters.extractOpenStreetMapOrgCoordinates
import net.retiolus.osm2gmaps.utils.maps.converters.getOmapsAppShortCode
import net.retiolus.osm2gmaps.utils.maps.converters.getOsmGoShortCode
import net.retiolus.osm2gmaps.utils.maps.extractHttpsLink
import net.retiolus.osm2gmaps.utils.maps.getMapuLink
import net.retiolus.osm2gmaps.utils.maps.handleNoValidLocationData

class MainActivity : BaseActivity() {
    private lateinit var inputText: EditText
    private lateinit var convertedLinksRecyclerView: RecyclerView
    private lateinit var convertedLinksAdapter: ConvertedLinksAdapter
    private lateinit var sharedPreferences: SharedPreferences
    private var locationInfo: LocationInfo? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        initializeViews()
        setUpToolbar()
        setUpLocationButton()
        setUpConvertButton()
        setUpFooterLinks()
        handleIntent()

        PreferencesUtil.setupGeneralPreferences(this)
        PreferencesUtil.setupDisplayedLinksPreferences(this)
    }

    private fun initializeViews() {
        inputText = findViewById(R.id.inputText)
        convertedLinksRecyclerView = findViewById(R.id.convertedLinksRecyclerView)
        sharedPreferences = getSharedPreferences("osm2gmaps", MODE_PRIVATE)
    }

    private fun setUpLocationButton() {
        val locationButton = findViewById<Button>(R.id.locationButton)
        locationButton.setOnClickListener {
            getLocation()
        }
    }

    private fun setUpConvertButton() {
        val convertButton = findViewById<Button>(R.id.convertButton)
        convertButton.setOnClickListener {
            val userInput = inputText.text.toString()

            convertButton.isEnabled = false
            val originalButtonText = convertButton.text.toString()
            val loadingText = ""
            val loadingAnimationJob = CoroutineScope(Dispatchers.Main).launch {
                while (true) {
                    convertButton.text = loadingText
                    delay(500)
                    convertButton.text = "."
                    delay(500)
                    convertButton.text = ".."
                    delay(500)
                    convertButton.text = "..."
                    delay(500)
                }
            }

            CoroutineScope(Dispatchers.Main).launch {
                val coords = handleUserInput(userInput)
                val convertedLinks = convertUserInput(coords.first, coords.second, coords.third)
                val saveHistoryValue = sharedPreferences.getBoolean("saveHistory", true)

                if (saveHistoryValue) {
                    locationInfo?.let { addToHistory(it) }
                }

                loadingAnimationJob.cancel()
                withContext(Dispatchers.Main){
                    displayConvertedLinks(convertedLinks)
                    convertButton.text = originalButtonText
                    convertButton.isEnabled = true
                }
            }
        }
    }

    @OptIn(DelicateCoroutinesApi::class)
    private fun handleIntent() {
        GlobalScope.launch {
            when (intent?.action) {
                Intent.ACTION_SEND -> handleActionSend(intent)
                Intent.ACTION_VIEW -> handleActionView(intent)
            }
        }
    }

    private suspend fun handleActionSend(intent: Intent) {
        val sharedText = intent.getStringExtra(Intent.EXTRA_TEXT)
        handleIntentData(sharedText)
    }

    private suspend fun handleActionView(intent: Intent) {
        val sharedText = intent.dataString
        handleIntentData(sharedText)
    }

    private suspend fun handleIntentData(sharedText: String?) {
        sharedText ?: return

        val coords = handleSharedText(sharedText)
        val selectedProviderIndex = sharedPreferences.getInt("selectedMapProviderIndex", 0)
        val link = when (selectedProviderIndex) {
            0 -> create2GisLink(coords.first, coords.second, coords.third)
            1 -> createAcmeMapperLink(coords.first, coords.second, coords.third)
            2 -> createAppleMapsLink(coords.first, coords.second, coords.third)
            3 -> createBingMapsLink(coords.first, coords.second, coords.third)
            4 -> createCopernixLink(coords.first, coords.second, coords.third)
            5 -> createGeoHackLink(coords.first, coords.second, coords.third)
            6 -> createGeoLink(coords.first, coords.second, coords.third)
            7 -> createGoogleMapsLink(coords.first, coords.second, coords.third)
            8 -> createHereWeGoLink(coords.first, coords.second, coords.third)
            9 -> createMagicEarthLink(coords.first, coords.second, coords.third)
            10 -> createMapQuestLink(coords.first, coords.second, coords.third)
            11 -> createOpenHistoricalMapLink(coords.first, coords.second, coords.third)
            12 -> createOpenStreetMapLink(coords.first, coords.second, coords.third)
            13 -> createOpenTopoMapLink(coords.first, coords.second, coords.third)
            14 -> createOmapsAppLink(coords.first, coords.second, coords.third)
            15 -> createWazeLink(coords.first, coords.second, coords.third)
            16 -> createWikiMapiaConverterLink(coords.first, coords.second, coords.third)
            17 -> createWikiMiniAtlasLink(coords.first, coords.second, coords.third)
            18 -> createYandexMapsLink(coords.first, coords.second, coords.third)
            else -> ""
        }
        val convertedLinks = convertUserInput(coords.first, coords.second, coords.third)
        val copyToClipboard = if (intent.action == Intent.ACTION_SEND) {
            sharedPreferences.getBoolean("copyToClipboardOnShare", false)
        } else {
            sharedPreferences.getBoolean("copyToClipboardOnOpenWith", false)
        }
        val openShareMenu = if (intent.action == Intent.ACTION_SEND) {
            sharedPreferences.getBoolean("openShareMenuOnShare", false)
        } else {
            sharedPreferences.getBoolean("openShareMenuOnOpenWith", false)
        }
        val openDefaultApp = if (intent.action == Intent.ACTION_SEND) {
            sharedPreferences.getBoolean("openDefaultAppOnShare", false)
        } else {
            sharedPreferences.getBoolean("openDefaultAppOnOpenWith", false)
        }
        displayConvertedLinks(convertedLinks)
        if (copyToClipboard) {
            copyToClipboard(link)
        }
        if (openShareMenu) {
            openSendMenuWithLink(this@MainActivity, link)
        }
        if (openDefaultApp){
            openDefaultApp(this@MainActivity, link)
        }

        val saveHistoryValue = sharedPreferences.getBoolean("saveHistory", true)
        if (saveHistoryValue) {
            locationInfo?.let { addToHistory(it) }
        }

    }

    private suspend fun handleUserInput(userInput: String): Triple<Double, Double, Int> {
        return handleSharedText(userInput)
    }

    private suspend fun handleSharedText(sharedText: String): Triple<Double, Double, Int> {
        if (sharedText == "") {
            runOnUiThread {
                Toast.makeText(
                    this@MainActivity,
                    R.string.toast_empty_input,
                    Toast.LENGTH_SHORT
                ).show()
            }
            return Triple(0.0, 0.0, 0)
        }

        val coordinates = when {
            sharedText.contains("2gis") -> handle2gisLink(sharedText)
            sharedText.startsWith("https://osm.org/go/") -> handleOsmOrgGoLink(sharedText)
            sharedText.contains("https://omaps.app/") -> handleOmapsAppLink(sharedText)
            sharedText.contains("openstreetmap.org/") -> extractOpenStreetMapOrgCoordinates(
                extractHttpsLink(sharedText)
            )

            sharedText.contains("https://maps.apple.com/") -> extractAppleMapsCoordinates(
                extractHttpsLink(sharedText)
            )

            sharedText.contains("bing.com") -> extractBingMapsCoordinates(
                extractHttpsLink(sharedText)
            )

            sharedText.contains("https://magicearth.com/") -> extractMagicEarhCoordinates(
                extractHttpsLink(sharedText)
            )

            sharedText.contains("waze.com") || sharedText.contains("yandex") || isGoogleMapsLink(sharedText) -> {
                val fullLink = handleProprietaryShortLink(sharedText)
                handleNoValidLocationData(extractHttpsLink(fullLink))
            }

            else -> handleNoValidLocationData(sharedText)
        }

        locationInfo = LocationInfo(
            source = sharedText,
            latitude = coordinates.first,
            longitude = coordinates.second,
            zoom = coordinates.third,
            timestamp = System.currentTimeMillis()
        )

        return coordinates
    }

    private fun isGoogleMapsLink(link: String): Boolean {
        val googleMapsLinks = listOf(
            "https://maps.app.goo.gl/",
            "https://goo.gl/",
            "https://www.google.com/maps/"
        )
        return googleMapsLinks.any { link.contains(it) }
    }

    private suspend fun handleProprietaryShortLink(link: String): String {
        val selectedProprietaryUnshortenerIndex =
            sharedPreferences.getInt("selectedProprietaryLinksUnshortener", 0)
        return when (selectedProprietaryUnshortenerIndex) {
            0 -> {
                runOnUiThread {
                    Toast.makeText(
                        this@MainActivity,
                        R.string.toast_need_mapu,
                        Toast.LENGTH_SHORT
                    ).show()
                }
                ""
            }
            1 -> {
                val coords = getMapuLink(link, "https://mapu.retiolus.net")
                if ("Error" in coords) {
                    runOnUiThread {
                        Toast.makeText(
                            this@MainActivity,
                            coords,
                            Toast.LENGTH_SHORT
                        ).show()
                    }
                }
                coords
            }
            else -> {
                val customResolver = sharedPreferences.getString("customGoogleMapsShortLink", "")
                if (customResolver.isNullOrEmpty()) {
                    runOnUiThread {
                        Toast.makeText(
                            this@MainActivity,
                            R.string.toast_need_custom_resolver,
                            Toast.LENGTH_SHORT
                        ).show()
                    }
                    ""
                } else {
                    val coords = getMapuLink(link, "https://$customResolver")
                    if ("Error" in coords) {
                        runOnUiThread {
                            Toast.makeText(
                                this@MainActivity,
                                coords,
                                Toast.LENGTH_SHORT
                            ).show()
                        }
                    }
                    coords
                }
            }
        }
    }

    private suspend fun handle2gisLink(sharedText: String): Triple<Double, Double, Int> {
        val extractedCoordinates = extract2GisCoordinates(extractHttpsLink(sharedText))
        return if (extractedCoordinates.first != 0.0) {
            extractedCoordinates
        } else {
            val link = handleProprietaryShortLink(sharedText)
            extract2GisCoordinates(link)
        }
    }

    private fun handleOsmOrgGoLink(link: String): Triple<Double, Double, Int> {
        return decodeOsmGoShortCode(getOsmGoShortCode(link))
    }

    private fun handleOmapsAppLink(link: String): Triple<Double, Double, Int> {
        return decodeOmapsAppShortCode(getOmapsAppShortCode(extractHttpsLink(link)))
    }

    private fun copyToClipboard(text: String) {
        val clipboardManager = getSystemService(CLIPBOARD_SERVICE) as ClipboardManager
        val clip = ClipData.newPlainText("Google Maps Link", text)
        clipboardManager.setPrimaryClip(clip)
    }

    private fun openSendMenuWithLink(context: Context, link: String) {
        val intent = Intent(Intent.ACTION_SEND)
        intent.type = "text/plain"
        intent.putExtra(Intent.EXTRA_TEXT, link)
        context.startActivity(Intent.createChooser(intent, "Share via"))
    }

    private fun openDefaultApp(context: Context, link: String) {
        val intent = Intent(Intent.ACTION_VIEW)
        intent.data = Uri.parse(link)
        context.startActivity(intent)
    }

    private fun convertUserInput(
        latitude: Double,
        longitude: Double,
        zoomLevel: Int
    ): List<LinkWithTitle> {
        val displayAppleMapsMain = sharedPreferences.getBoolean("displayAppleMapsMain", false)
        val displayBingMapsMain = sharedPreferences.getBoolean("displayBingMapsMain", false)
        val displayCopernixMain = sharedPreferences.getBoolean("displayCopernixMain", false)
        val displayDMSMain = sharedPreferences.getBoolean("displayDMSMain", false)
        val displayGeoHackMain = sharedPreferences.getBoolean("displayGeoHackMain", false)
        val displayGeoLinkMain = sharedPreferences.getBoolean("displayGeoLinkMain", false)
        val displayGoogleMapsMain = sharedPreferences.getBoolean("displayGoogleMapsMain", false)
        val displayHereWeGoMain = sharedPreferences.getBoolean("displayHereWeGoMain", false)
        val displayMagicEarthMain = sharedPreferences.getBoolean("displayMagicEarthMain", false)
        val displayMapQuestMain = sharedPreferences.getBoolean("displayMapQuestMain", false)
        val displayOpenHistoricalMapMain =
            sharedPreferences.getBoolean("displayOpenHistoricalMapMain", false)
        val displayOpenStreetMapMain =
            sharedPreferences.getBoolean("displayOpenStreetMapMain", false)
        val displayOpenTopoMapMain = sharedPreferences.getBoolean("displayOpenTopoMapMain", false)
        /*val displayOmapsAppMain = sharedPreferences.getBoolean("displayOmapsAppMain", false)*/
        val displayOrganicMapsMain = sharedPreferences.getBoolean("displayOrganicMapsMain", false)
        val displayWazeMain = sharedPreferences.getBoolean("displayWazeMain", false)
        val displayWikiMapiaMain = sharedPreferences.getBoolean("displayWikiMapiaMain", false)
        val displayWikiMiniAtlasMain =
            sharedPreferences.getBoolean("displayWikiMiniAtlasMain", false)
        val displayYandexMain = sharedPreferences.getBoolean("displayYandexMain", false)

        val linksWithTitles = mutableListOf<LinkWithTitle>()

        if (displayGeoLinkMain) {
            val geoLink = createGeoLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("GeoLink", geoLink)
            linksWithTitles.add(LinkWithTitle(geoLink, "GeoLink"))
        }

        if (displayAppleMapsMain) {
            val appleMapsLink = createAppleMapsLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("Apple Maps", appleMapsLink)
            linksWithTitles.add(LinkWithTitle(appleMapsLink, "Apple Maps"))
        }

        if (displayBingMapsMain) {
            val bingMapsLink = createBingMapsLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("Bing Maps", bingMapsLink)
            linksWithTitles.add(LinkWithTitle(bingMapsLink, "Bing Maps"))
        }

        if (displayCopernixMain) {
            val copernixLink = createCopernixLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("Copernix", copernixLink)
            linksWithTitles.add(LinkWithTitle(copernixLink, "Copernix"))
        }

        if (displayDMSMain) {
            val dmsLink = createDMSLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("DMS", dmsLink)
            linksWithTitles.add(LinkWithTitle(dmsLink, "DMS"))
        }

        if (displayGeoHackMain) {
            val geoHackLink = createGeoHackLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("GeoHack", geoHackLink)
            linksWithTitles.add(LinkWithTitle(geoHackLink, "GeoHack"))
        }

        if (displayGoogleMapsMain) {
            val googleMapsLink = createGoogleMapsLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("Google Maps", googleMapsLink)
            linksWithTitles.add(LinkWithTitle(googleMapsLink, "Google Maps"))
        }

        if (displayHereWeGoMain) {
            val hereWeGoLink = createHereWeGoLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("HERE WeGo", hereWeGoLink)
            linksWithTitles.add(LinkWithTitle(hereWeGoLink, "HERE WeGo"))
        }

        if (displayMagicEarthMain) {
            val magicEarthLink = createMagicEarthLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("Magic Earth", magicEarthLink)
            linksWithTitles.add(LinkWithTitle(magicEarthLink, "Magic Earth"))
        }

        if (displayMapQuestMain) {
            val mapQuestLink = createMapQuestLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("MapQuest", mapQuestLink)
            linksWithTitles.add(LinkWithTitle(mapQuestLink, "MapQuest"))
        }

        if (displayOpenHistoricalMapMain) {
            val openHistoricalMapLink = createOpenHistoricalMapLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("OpenHistoricalMap", openHistoricalMapLink)
            linksWithTitles.add(LinkWithTitle(openHistoricalMapLink, "OpenHistoricalMap"))
        }

        if (displayOpenStreetMapMain) {
            val openStreetMapLink = createOpenStreetMapLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("OpenStreetMap", openStreetMapLink)
            linksWithTitles.add(LinkWithTitle(openStreetMapLink, "OpenStreetMap"))
        }

        if (displayOpenTopoMapMain) {
            val openTopoMapLink = createOpenTopoMapLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("OpenTopoMap", openTopoMapLink)
            linksWithTitles.add(LinkWithTitle(openTopoMapLink, "OpenTopoMap"))
        }

        /*if (displayOmapsAppMain) {
            val omapsAppLink = createOmapsAppLink(latitude, longitude, zoomLevel)
            val organicMapsLink = createOrganicMapsLink(latitude, longitude, zoomLevel)
            linksWithTitles.add(LinkWithTitle(omapsAppLink, "Organic Maps"))
            linksWithTitles.add(LinkWithTitle(organicMapsLink, "Organic Maps"))
        }*/

        if (displayOrganicMapsMain) {
            val omapsAppLink = createOmapsAppLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("Omaps App", omapsAppLink)
            /*val organicMapsLink = createOrganicMapsLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("Organic Maps", organicMapsLink)*/
            linksWithTitles.add(LinkWithTitle(omapsAppLink, "Organic Maps"))
            /*linksWithTitles.add(LinkWithTitle(organicMapsLink, "Organic Maps"))*/
        }

        if (displayWazeMain) {
            val wazeLink = createWazeLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("Waze", wazeLink)
            linksWithTitles.add(LinkWithTitle(wazeLink, "Waze"))
        }

        if (displayWikiMapiaMain) {
            val wikiMapiaLink = createWikiMapiaConverterLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("Wikimapia", wikiMapiaLink)
            linksWithTitles.add(LinkWithTitle(wikiMapiaLink, "Wikimapia"))
        }

        if (displayWikiMiniAtlasMain) {
            val wikiMiniAtlasLink = createWikiMiniAtlasLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("WikiMiniAtlas", wikiMiniAtlasLink)
            linksWithTitles.add(LinkWithTitle(wikiMiniAtlasLink, "WikiMiniAtlas"))
        }

        if (displayYandexMain) {
            val yandexLink = createYandexMapsLink(latitude, longitude, zoomLevel)
            locationInfo?.addMapLink("Yandex Maps", yandexLink)
            linksWithTitles.add(LinkWithTitle(yandexLink, "Yandex Maps"))
        }

        val customLinksSharedPreferences = getSharedPreferences("custom_links", Context.MODE_PRIVATE)
        customLinksSharedPreferences.all.forEach { (name, link) ->
            if (link is String) {
                val customLink = link.replace("{longitude}", longitude.toString())
                    .replace("{latitude}", latitude.toString())
                linksWithTitles.add(LinkWithTitle(customLink, name))
            }
        }

        println(linksWithTitles)
        return linksWithTitles
    }

    private fun displayConvertedLinks(convertedLinks: List<LinkWithTitle>) {
        val links = convertedLinks.map { it.link }
        val titles = convertedLinks.map { it.title }

        runOnUiThread {
            if (!::convertedLinksAdapter.isInitialized) {
                convertedLinksAdapter = ConvertedLinksAdapter(links, titles)
                convertedLinksRecyclerView.layoutManager = LinearLayoutManager(this)
                convertedLinksRecyclerView.adapter = convertedLinksAdapter
            } else {
                convertedLinksAdapter.updateData(links, titles)
            }
        }
    }

    private fun getLocation() {
        if (ContextCompat.checkSelfPermission(
                this,
                android.Manifest.permission.ACCESS_FINE_LOCATION
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            ActivityCompat.requestPermissions(
                this,
                arrayOf(android.Manifest.permission.ACCESS_FINE_LOCATION),
                LOCATION_PERMISSION_REQUEST_CODE
            )
            Log.d("Location", "Permission not granted")
            return
        }

        val locationManager = getSystemService(Context.LOCATION_SERVICE) as LocationManager

        try {
            var location: Location? = null

            if (locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
                location = locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER)
            }

            if (location == null && locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
                location = locationManager.getLastKnownLocation(LocationManager.NETWORK_PROVIDER)
            }

            if (location != null) {
                val inputEditText = findViewById<TextInputEditText>(R.id.inputText)
                inputEditText.setText("${location.latitude}, ${location.longitude}")
                Log.d("Location", "Location found: ${location.latitude}, ${location.longitude}")
            } else {
                Log.d("Location", "Location is null")
            }
        } catch (ex: SecurityException) {
            Log.e("Location", "Security Exception: ${ex.message}")
        }
    }


    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<out String>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (requestCode == LOCATION_PERMISSION_REQUEST_CODE) {
            if (grantResults.isNotEmpty() && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                getLocation()
            }
        }
    }

    companion object {
        private const val LOCATION_PERMISSION_REQUEST_CODE = 100
    }

    private fun addToHistory(locationInfo: LocationInfo) {
        GlobalScope.launch {
            val gson = Gson()
            val historyJson = sharedPreferences.getString("history", "[]") ?: "[]"
            Log.d("HistoryActivity", "History JSON: $historyJson")
            val historyListType = object : TypeToken<ArrayList<LocationInfo>>() {}.type
            val historyList: ArrayList<LocationInfo> = gson.fromJson(historyJson, historyListType)

            val existingItemIndex = historyList.indexOfFirst { it.source == locationInfo.source }

            if (existingItemIndex != -1) {
                historyList.removeAt(existingItemIndex)
            }

            historyList.add(locationInfo)

            val updatedHistoryJson = gson.toJson(historyList)
            Log.d("HistoryActivity", "Updated History JSON: $updatedHistoryJson")
            sharedPreferences.edit().putString("history", updatedHistoryJson).apply()
        }
    }

}
